"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ajv_1 = require("ajv");
const _2019_1 = require("ajv/dist/2019");
const options_1 = require("./options");
const util = require("./util");
const path = require("path");
const draft6metaSchema = require("ajv/lib/refs/json-schema-draft-06.json");
function default_1(argv) {
    const opts = options_1.getOptions(argv);
    if (argv.o)
        opts.code.source = true;
    const Ajv = argv.spec === "draft2019" ? _2019_1.default : ajv_1.default;
    const ajv = new Ajv(opts);
    let invalid;
    ajv.addMetaSchema(draft6metaSchema);
    addSchemas(argv.m, "addMetaSchema", "meta-schema");
    addSchemas(argv.r, "addSchema", "schema");
    customFormatsKeywords(argv.c);
    if (invalid)
        process.exit(1);
    return ajv;
    function addSchemas(args, method, fileType) {
        if (!args)
            return;
        const files = util.getFiles(args);
        files.forEach((file) => {
            const schema = util.openFile(file, fileType);
            try {
                ajv[method](schema);
            }
            catch (err) {
                console.error(`${fileType} ${file} is invalid`);
                console.error(`error: ${err.message}`);
                invalid = true;
            }
        });
    }
    function customFormatsKeywords(args) {
        if (!args)
            return;
        const files = util.getFiles(args);
        files.forEach((file) => {
            if (file[0] === ".")
                file = path.resolve(process.cwd(), file);
            try {
                require(file)(ajv);
            }
            catch (err) {
                console.error(`module ${file} is invalid; it should export function`);
                console.error(`error: ${err.message}`);
                invalid = true;
            }
        });
    }
}
exports.default = default_1;
//# sourceMappingURL=ajv.js.map